@@ -151,6 +151,15 @@ class Future(WrappedKey):
         """
         return sync(self.client.loop, self._exception)
 
+    def add_done_callback(self, fn):
+        """ Call callback on future when callback has finished
+
+        The callback ``fn`` should take the future as its only argument.  This
+        will be called regardless of if the future completes successfully,
+        errs, or is cancelled
+        """
+        self.client.loop.add_callback(done_callback, self, fn)
+
     def cancel(self):
         """ Returns True if the future has been cancelled """
         return self.client.cancel([self])
@@ -225,6 +234,14 @@ class Future(WrappedKey):
     __repr__ = __str__
 
 
+@gen.coroutine
+def done_callback(future, callback):
+    """ Wait on future, then call callback """
+    while future.status == 'pending':
+        yield future.event.wait()
+    callback(future)
+
+
 @partial(normalize_token.register, Future)
 def normalize_future(f):
     return [f.key, type(f)]
@@ -521,16 +538,24 @@ class Client(object):
                 self.scheduler.close_rpc()
 
     def shutdown(self, timeout=10):
-        """ Send shutdown signal and wait until scheduler terminates """
+        """ Send shutdown signal and wait until scheduler terminates
+
+        This cancels all currently running tasks, clears the state of the
+        scheduler, and shuts down all workers and scheduler.
+
+        You do not need to call this when you finish your session.  You only
+        need to call this if you want to take down the distributed cluster.
+
+        See Also
+        --------
+        Client.restart
+        """
+        # XXX handling of self.status here is not thread-safe
         if self.status == 'closed':
             return
-        self.status = 'closed'
-        with ignoring(AttributeError):
-            self.loop.add_callback(self.scheduler_stream.send,
-                                   {'op': 'close-stream'})
-            sync(self.loop, self.scheduler_stream.close)
-        with ignoring(AttributeError):
-            self.scheduler.close_rpc()
+        sync(self.loop, self._shutdown, fast=True)
+        assert self.status == 'closed'
+
         if self._should_close_loop:
             sync(self.loop, self.loop.stop)
             self.loop.close(all_fds=True)
@@ -539,8 +564,6 @@ class Client(object):
             dask.set_options(get=self._previous_get)
         with ignoring(AttributeError):
             dask.set_options(shuffle=self._previous_shuffle)
-        if _global_client[0] is self:
-            _global_client[0] = None
         if self.get == _globals.get('get'):
             del _globals['get']
         with ignoring(AttributeError):
@@ -700,7 +723,7 @@ class Client(object):
         else:
             uid = str(uuid.uuid4())
             keys = [funcname(func) + '-' + uid + '-' + str(i)
-                    for i in range(min(map(len, iterables)))]
+                    for i in range(min(map(len, iterables)))] if iterables else []
 
         if not kwargs:
             dsk = {key: (func,) + args
@@ -921,6 +944,11 @@ class Client(object):
     def scatter(self, data, workers=None, broadcast=False, maxsize=0):
         """ Scatter data into distributed memory
 
+        This moves data from the local client process into the workers of the
+        distributed scheduler.  Note that it is often better to submit jobs to
+        your workers to have them load the data rather than loading data
+        locally and then scattering it out to them.
+
         Parameters
         ----------
         data: list, iterator, dict, or Queue
@@ -952,14 +980,17 @@ class Client(object):
          'z': <Future: status: finished, key: z>}
 
         Constrain location of data to subset of workers
+
         >>> c.scatter([1, 2, 3], workers=[('hostname', 8788)])   # doctest: +SKIP
 
         Handle streaming sequences of data with iterators or queues
+
         >>> seq = c.scatter(iter([1, 2, 3]))  # doctest: +SKIP
         >>> next(seq)  # doctest: +SKIP
         <Future: status: finished, key: c0a8a20f903a4915b94db8de3ea63195>,
 
         Broadcast data to all workers
+
         >>> [future] = c.scatter([element], broadcast=True)  # doctest: +SKIP
 
         See Also
@@ -1034,11 +1065,13 @@ class Client(object):
         Examples
         --------
         Publishing client:
+
         >>> df = dd.read_csv('s3://...')  # doctest: +SKIP
         >>> df = c.persist(df) # doctest: +SKIP
         >>> c.publish_dataset(my_dataset=df)  # doctest: +SKIP
 
         Receiving client:
+
         >>> c.list_datasets()  # doctest: +SKIP
         ['my_dataset']
         >>> df2 = c.get_dataset('my_dataset')  # doctest: +SKIP
@@ -1582,11 +1615,12 @@ class Client(object):
     def replicate(self, futures, n=None, workers=None, branching_factor=2):
         """ Set replication of futures within network
 
-        This performs a tree copy of the data throughout the network
-        individually on each piece of data.
+        Copy data onto many workers.  This helps to broadcast frequently
+        accessed data and it helps to improve resilience.
 
-        This operation blocks until complete.  It does not guarantee
-        replication of data to future workers.
+        This performs a tree copy of the data throughout the network
+        individually on each piece of data.  This operation blocks until
+        complete.  It does not guarantee replication of data to future workers.
 
         Parameters
         ----------
