@@ -73,7 +73,7 @@ def loads(x):
     try:
         return pickle.loads(x)
     except Exception:
-        logger.info("Failed to deserialize %s", x, exc_info=True)
+        logger.info("Failed to deserialize %s", x[:10000], exc_info=True)
         raise
 
 
@@ -131,11 +131,12 @@ class Server(TCPServer):
     """
     default_port = 0
 
-    def __init__(self, handlers, max_buffer_size=MAX_BUFFER_SIZE, **kwargs):
+    def __init__(self, handlers, max_buffer_size=MAX_BUFFER_SIZE,
+            connection_limit=512, **kwargs):
         self.handlers = assoc(handlers, 'identity', self.identity)
         self.id = str(uuid.uuid1())
         self._port = None
-        self.rpc = ConnectionPool()
+        self.rpc = ConnectionPool(limit=connection_limit)
         super(Server, self).__init__(max_buffer_size=max_buffer_size, **kwargs)
 
     @property
@@ -394,6 +395,8 @@ class rpc(object):
 
     >>> remote.close_streams()  # doctest: +SKIP
     """
+    active = 0
+
     def __init__(self, arg=None, stream=None, ip=None, port=None, addr=None,
             timeout=3):
         ip, port = ip_port_from_args(arg=arg, addr=addr, ip=ip, port=port)
@@ -402,6 +405,7 @@ class rpc(object):
         self.port = port
         self.timeout = timeout
         self.status = 'running'
+        rpc.active += 1
         assert self.ip
         assert self.port
 
@@ -451,6 +455,7 @@ class rpc(object):
                     stream.close()
                 except (OSError, IOError, StreamClosedError):
                     pass
+        self.streams.clear()
 
     def __getattr__(self, key):
         @gen.coroutine
@@ -461,13 +466,18 @@ class rpc(object):
             raise gen.Return(result)
         return send_recv_from_rpc
 
-    def __del__(self):
-        self.close_streams()
-
     def close_rpc(self):
+        if self.status != 'closed':
+            rpc.active -= 1
         self.status = 'closed'
         self.close_streams()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, *args):
+        self.close_rpc()
+
 
 class RPCCall(object):
     """ The result of ConnectionPool()('host:port')
@@ -495,6 +505,9 @@ class RPCCall(object):
             raise gen.Return(result)
         return send_recv_from_rpc
 
+    def close_rpc(self):
+        pass
+
 
 class ConnectionPool(object):
     """ A maximum sized pool of Tornado IOStreams
@@ -580,7 +593,15 @@ class ConnectionPool(object):
     def collect(self):
         logger.info("Collecting unused streams.  open: %d, active: %d",
                     self.open, self.active)
-        for k, streams in list(self.available.items()):
+        for streams in list(self.available.values()):
+            for stream in streams:
+                stream.close()
+
+    def close(self):
+        for streams in list(self.available.values()):
+            for stream in streams:
+                stream.close()
+        for streams in list(self.occupied.values()):
             for stream in streams:
                 stream.close()
 
